#!/app/local/bin/virtual-python
#$#HEADER-START
# vim:set expandtab ts=4 sw=4 ai ft=python:
#
#     Reflex Configuration Event Engine
#
#     Copyright (C) 2016 Brandon Gillespie
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU Affero General Public License as published
#     by the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU Affero General Public License for more details.
#
#     You should have received a copy of the GNU Affero General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#$#HEADER-END

import sys
import rfx.client
import dictlib
import requests.exceptions

################################################################################
def create(func, otype, odata):
    """Wrapper to handle errors"""
    try:
        sys.stdout.write("Create " + otype + "." + odata.get('name'))
        res = do(func, otype, odata)
        if res.get('status') != 'created':
            msg = res.get('message')
            print(" - " + res.get('message'))
        else:
            print("")
        return dictlib.Obj(res)

    except rfx.client.ClientError as err:
        errmsg = str(err)
        if "already exists" in errmsg or "Duplicate entry" in errmsg:
            print(" - already exists")
            return False
        else:
            print(" - error:\n       " + errmsg)
            if 'Unable to authorize session' in errmsg:
                sys.exit(0)

def do(func, *args):
    """Wrapper to handle errors"""
    try:
        return func(*args)
    except requests.exceptions.ConnectionError:
        print("Server not available at " + rcs.cfg['REFLEX_URL'])
        sys.exit(1)

def setup():
    rcs = rfx.client.Session()
    rcs.cfg_load()
    data = dictlib.Obj()

    ########################################
    # master config for reflex tools
    create(rcs.create, "config", {
        "name":"reflex",
        "type":"parameter",
        "config": {
            "regions": {
                "local": {
                    "lanes": [
                        "dev",
                        "prd",
                        "stg"
                    ]
                }
            },
            "lanes": {
                "dev": {
                    "short": "d",
                    "full": "Development"
                },
                "prd": {
                    "short": "p",
                    "full": "Production"
                },
                "stg": {
                    "short": "s",
                    "full": "Staging"
                }
            }
        }
    })
    ########################################
    create(rcs.create, "apikey", {
        "name":"svc-juno"
    })
    create(rcs.create, "apikey", {
        "name":"brandon"
    })

    data['keys'] = dictlib.Obj({
        'svc-juno': do(rcs.get, "apikey", "svc-juno"),
        'svc-juno': do(rcs.get, "apikey", "brandon"),
    })

    ########################################
    create(rcs.create, "group", {
        "name":"svc-juno",
        "group":["svc-juno"],
        "type":"Apikey"
    })

    create(rcs.create, "group", {
        "name":"admin",
        "group":["brandon"],
        "type":"Apikey"
    })

    create(rcs.create, "policy", {
        "name": "admin-all-access",
        "policy": "token_name in groups.admin"
    })

    create(rcs.create, "policy", {
        "name": "svc-juno",
        "policy": "token_name in groups.svc_juno and sensitive == False"
    })

    data['policies'] = dictlib.Obj({
        'svc-juno': do(rcs.get, "policy", "svc-juno"),
        'admin': do(rcs.get, "policy", "admin")
    })

    ########################################
    create(rcs.create, "policyscope", {
        "name": "juno",
        "policy_id": data.policies.support_not_sensitive.id,
        "actions": 'read',
        "type": 'global',
        "matches": 'obj_type == "Service" and re.match(r"^juno", obj["name"])'
    })
    create(rcs.create, "policyscope", {
        "name": "admin",
        "policy_id": data.policies.support_not_sensitive.id,
        "actions": 'admin',
        "type": 'global',
        "matches": 'True'
    })

    ########################################
    create(rcs.create, "pipeline", {
        "name": "juno",
        "title": "Juno",
    })

    create(rcs.create, "config", {
        "name": "juno",
        "type": "parameter",
        "setenv": {
            "PORT": 4000
        }
    })
    create(rcs.create, "config", {
        "name": "juno-d1",
        "type": "parameter",
        "extends": ["juno"],
        "sensitive": {
           "parameters": {
           }
        }
    })

    create(rcs.create, "config", {
        "name": "juno-s1",
        "type": "parameter",
        "extends": ["juno"],
        "sensitive": {
           "parameters": {
           }
        }
    })
    create(rcs.create, "config", {
        "name": "juno-p1",
        "type": "parameter",
        "extends": ["juno"],
        "sensitive": {
           "parameters": {
           }
        }
    })

    create(rcs.create, "service", {
        "name": "juno-d1",
        "config": "juno-d1",
        "pipeline": "juno",
    })

    create(rcs.create, "service", {
        "name": "juno-s1",
        "config": "juno-s1",
        "pipeline": "juno",
    })

    create(rcs.create, "service", {
        "name": "juno-p1",
        "config": "juno-p1",
        "pipeline": "juno",
    })

